# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.metadata import Metadata

__all__ = ["Memory"]


class Memory(BaseModel):
    """Response model for the GET /memories/get endpoint."""

    resource_id: str

    source: Literal[
        "reddit",
        "notion",
        "slack",
        "google_calendar",
        "google_mail",
        "box",
        "dropbox",
        "google_drive",
        "github",
        "vault",
        "web_crawler",
        "trace",
        "microsoft_teams",
    ]

    type: str
    """The type of document (e.g. Document, Website, Email)"""

    data: Optional[List[object]] = None
    """The structured content of the document"""

    memories: Optional[List[str]] = None
    """Summaries of all memories extracted from this document"""

    metadata: Optional[Metadata] = None

    title: Optional[str] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["MemoryListParams"]


class MemoryListParams(TypedDict, total=False):
    collection: Optional[str]
    """Filter documents by collection."""

    cursor: Optional[str]

    filter: Optional[str]
    """Filter documents by metadata using MongoDB-style operators.

    Example: {"department": "engineering", "priority": {"$gt": 3}}
    """

    size: int

    source: Optional[
        Literal[
            "reddit",
            "notion",
            "slack",
            "google_calendar",
            "google_mail",
            "box",
            "dropbox",
            "google_drive",
            "github",
            "vault",
            "web_crawler",
            "trace",
            "microsoft_teams",
        ]
    ]
    """Filter documents by source."""

    status: Optional[Literal["pending", "processing", "completed", "failed"]]
    """Filter documents by status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .notification import Notification

__all__ = ["Metadata"]


class Metadata(BaseModel):
    created_at: Optional[datetime] = None

    events: Optional[List[Notification]] = None

    indexed_at: Optional[datetime] = None

    last_modified: Optional[datetime] = None

    status: Optional[Literal["pending", "processing", "completed", "failed"]] = None

    url: Optional[str] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["AuthMeResponse", "App"]


class App(BaseModel):
    """The Hyperspell app's id this user belongs to"""

    id: str
    """The Hyperspell app's id this user belongs to"""

    icon_url: Optional[str] = None
    """The app's icon"""

    name: str
    """The app's name"""

    redirect_url: Optional[str] = None
    """The app's redirect URL"""


class AuthMeResponse(BaseModel):
    id: str
    """The user's id"""

    app: App
    """The Hyperspell app's id this user belongs to"""

    available_integrations: List[
        Literal[
            "reddit",
            "notion",
            "slack",
            "google_calendar",
            "google_mail",
            "box",
            "dropbox",
            "google_drive",
            "github",
            "vault",
            "web_crawler",
            "trace",
            "microsoft_teams",
        ]
    ]
    """All integrations available for the app"""

    installed_integrations: List[
        Literal[
            "reddit",
            "notion",
            "slack",
            "google_calendar",
            "google_mail",
            "box",
            "dropbox",
            "google_drive",
            "github",
            "vault",
            "web_crawler",
            "trace",
            "microsoft_teams",
        ]
    ]
    """All integrations installed for the user"""

    token_expiration: Optional[datetime] = None
    """The expiration time of the user token used to make the request"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .memory_status import MemoryStatus

__all__ = ["MemoryAddBulkResponse"]


class MemoryAddBulkResponse(BaseModel):
    """Response schema for successful bulk ingestion."""

    count: int
    """Number of items successfully processed"""

    items: List[MemoryStatus]
    """Status of each ingested item"""

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Notification"]


class Notification(BaseModel):
    message: str

    type: Literal["error", "warning", "info", "success"]

    time: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hyperspell import Hyperspell, AsyncHyperspell
from tests.utils import assert_matches_type
from hyperspell.types import MemoryStatus
from hyperspell._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSessions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_add(self, client: Hyperspell) -> None:
        session = client.sessions.add(
            history="history",
        )
        assert_matches_type(MemoryStatus, session, path=["response"])

    @parametrize
    def test_method_add_with_all_params(self, client: Hyperspell) -> None:
        session = client.sessions.add(
            history="history",
            date=parse_datetime("2019-12-27T18:11:19.117Z"),
            extract=["procedure"],
            format="vercel",
            metadata={"foo": "string"},
            session_id="session_id",
            title="title",
        )
        assert_matches_type(MemoryStatus, session, path=["response"])

    @parametrize
    def test_raw_response_add(self, client: Hyperspell) -> None:
        response = client.sessions.with_raw_response.add(
            history="history",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(MemoryStatus, session, path=["response"])

    @parametrize
    def test_streaming_response_add(self, client: Hyperspell) -> None:
        with client.sessions.with_streaming_response.add(
            history="history",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(MemoryStatus, session, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSessions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_add(self, async_client: AsyncHyperspell) -> None:
        session = await async_client.sessions.add(
            history="history",
        )
        assert_matches_type(MemoryStatus, session, path=["response"])

    @parametrize
    async def test_method_add_with_all_params(self, async_client: AsyncHyperspell) -> None:
        session = await async_client.sessions.add(
            history="history",
            date=parse_datetime("2019-12-27T18:11:19.117Z"),
            extract=["procedure"],
            format="vercel",
            metadata={"foo": "string"},
            session_id="session_id",
            title="title",
        )
        assert_matches_type(MemoryStatus, session, path=["response"])

    @parametrize
    async def test_raw_response_add(self, async_client: AsyncHyperspell) -> None:
        response = await async_client.sessions.with_raw_response.add(
            history="history",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(MemoryStatus, session, path=["response"])

    @parametrize
    async def test_streaming_response_add(self, async_client: AsyncHyperspell) -> None:
        async with async_client.sessions.with_streaming_response.add(
            history="history",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(MemoryStatus, session, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SessionAddParams"]


class SessionAddParams(TypedDict, total=False):
    history: Required[str]
    """The trace history as a string.

    Can be a JSON array of Hyperdoc steps, a JSON array of Vercel AI SDK steps, or
    OpenClaw JSONL.
    """

    date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Date of the trace"""

    extract: List[Literal["procedure", "memory"]]
    """What kind of memories to extract from the trace"""

    format: Optional[Literal["vercel", "hyperdoc", "openclaw"]]
    """Trace format: 'vercel', 'hyperdoc', or 'openclaw'. Auto-detected if not set."""

    metadata: Optional[Dict[str, Union[str, float, bool]]]
    """Custom metadata for filtering.

    Keys must be alphanumeric with underscores, max 64 chars.
    """

    session_id: str
    """Resource identifier for the trace."""

    title: Optional[str]
    """Title of the trace"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .metadata import Metadata as Metadata
from .resource import Resource as Resource
from .notification import Notification as Notification
from .query_result import QueryResult as QueryResult



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .token import Token as Token
from .memory import Memory as Memory
from .shared import Metadata as Metadata, Resource as Resource, QueryResult as QueryResult, Notification as Notification
from .memory_status import MemoryStatus as MemoryStatus
from .auth_me_response import AuthMeResponse as AuthMeResponse
from .memory_add_params import MemoryAddParams as MemoryAddParams
from .vault_list_params import VaultListParams as VaultListParams
from .memory_list_params import MemoryListParams as MemoryListParams
from .session_add_params import SessionAddParams as SessionAddParams
from .vault_list_response import VaultListResponse as VaultListResponse
from .memory_search_params import MemorySearchParams as MemorySearchParams
from .memory_update_params import MemoryUpdateParams as MemoryUpdateParams
from .memory_upload_params import MemoryUploadParams as MemoryUploadParams
from .auth_user_token_params import AuthUserTokenParams as AuthUserTokenParams
from .memory_add_bulk_params import MemoryAddBulkParams as MemoryAddBulkParams
from .memory_delete_response import MemoryDeleteResponse as MemoryDeleteResponse
from .memory_status_response import MemoryStatusResponse as MemoryStatusResponse
from .connection_list_response import ConnectionListResponse as ConnectionListResponse
from .memory_add_bulk_response import MemoryAddBulkResponse as MemoryAddBulkResponse
from .auth_delete_user_response import AuthDeleteUserResponse as AuthDeleteUserResponse
from .integration_list_response import IntegrationListResponse as IntegrationListResponse
from .action_add_reaction_params import ActionAddReactionParams as ActionAddReactionParams
from .action_send_message_params import ActionSendMessageParams as ActionSendMessageParams
from .connection_revoke_response import ConnectionRevokeResponse as ConnectionRevokeResponse
from .integration_connect_params import IntegrationConnectParams as IntegrationConnectParams
from .evaluate_score_query_params import EvaluateScoreQueryParams as EvaluateScoreQueryParams
from .action_add_reaction_response import ActionAddReactionResponse as ActionAddReactionResponse
from .action_send_message_response import ActionSendMessageResponse as ActionSendMessageResponse
from .integration_connect_response import IntegrationConnectResponse as IntegrationConnectResponse
from .evaluate_score_query_response import EvaluateScoreQueryResponse as EvaluateScoreQueryResponse
from .evaluate_score_highlight_params import EvaluateScoreHighlightParams as EvaluateScoreHighlightParams
from .evaluate_score_highlight_response import EvaluateScoreHighlightResponse as EvaluateScoreHighlightResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import session_add_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.memory_status import MemoryStatus

__all__ = ["SessionsResource", "AsyncSessionsResource"]


class SessionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SessionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return SessionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SessionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return SessionsResourceWithStreamingResponse(self)

    def add(
        self,
        *,
        history: str,
        date: Union[str, datetime] | Omit = omit,
        extract: List[Literal["procedure", "memory"]] | Omit = omit,
        format: Optional[Literal["vercel", "hyperdoc", "openclaw"]] | Omit = omit,
        metadata: Optional[Dict[str, Union[str, float, bool]]] | Omit = omit,
        session_id: str | Omit = omit,
        title: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryStatus:
        """
        Add an agent trace/transcript to the index.

        Accepts traces as a string in Hyperdoc format (native), Vercel AI SDK format, or
        OpenClaw JSONL format. The format is auto-detected if not specified.

        **Hyperdoc format** (JSON array, snake_case with type discriminators):

        ```json
        {
          "history": "[{\"type\": \"trace_message\", \"role\": \"user\", \"text\": \"Hello\"}]"
        }
        ```

        **Vercel AI SDK format** (JSON array, camelCase):

        ```json
        { "history": "[{\"role\": \"user\", \"content\": \"Hello\"}]" }
        ```

        **OpenClaw JSONL format** (newline-delimited JSON):

        ```json
        {
          "history": "{\"type\":\"session\",\"id\":\"abc\"}\n{\"type\":\"message\",\"message\":{\"role\":\"user\",...}}"
        }
        ```

        Args:
          history: The trace history as a string. Can be a JSON array of Hyperdoc steps, a JSON
              array of Vercel AI SDK steps, or OpenClaw JSONL.

          date: Date of the trace

          extract: What kind of memories to extract from the trace

          format: Trace format: 'vercel', 'hyperdoc', or 'openclaw'. Auto-detected if not set.

          metadata: Custom metadata for filtering. Keys must be alphanumeric with underscores, max
              64 chars.

          session_id: Resource identifier for the trace.

          title: Title of the trace

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/trace/add",
            body=maybe_transform(
                {
                    "history": history,
                    "date": date,
                    "extract": extract,
                    "format": format,
                    "metadata": metadata,
                    "session_id": session_id,
                    "title": title,
                },
                session_add_params.SessionAddParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryStatus,
        )


class AsyncSessionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSessionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncSessionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSessionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return AsyncSessionsResourceWithStreamingResponse(self)

    async def add(
        self,
        *,
        history: str,
        date: Union[str, datetime] | Omit = omit,
        extract: List[Literal["procedure", "memory"]] | Omit = omit,
        format: Optional[Literal["vercel", "hyperdoc", "openclaw"]] | Omit = omit,
        metadata: Optional[Dict[str, Union[str, float, bool]]] | Omit = omit,
        session_id: str | Omit = omit,
        title: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryStatus:
        """
        Add an agent trace/transcript to the index.

        Accepts traces as a string in Hyperdoc format (native), Vercel AI SDK format, or
        OpenClaw JSONL format. The format is auto-detected if not specified.

        **Hyperdoc format** (JSON array, snake_case with type discriminators):

        ```json
        {
          "history": "[{\"type\": \"trace_message\", \"role\": \"user\", \"text\": \"Hello\"}]"
        }
        ```

        **Vercel AI SDK format** (JSON array, camelCase):

        ```json
        { "history": "[{\"role\": \"user\", \"content\": \"Hello\"}]" }
        ```

        **OpenClaw JSONL format** (newline-delimited JSON):

        ```json
        {
          "history": "{\"type\":\"session\",\"id\":\"abc\"}\n{\"type\":\"message\",\"message\":{\"role\":\"user\",...}}"
        }
        ```

        Args:
          history: The trace history as a string. Can be a JSON array of Hyperdoc steps, a JSON
              array of Vercel AI SDK steps, or OpenClaw JSONL.

          date: Date of the trace

          extract: What kind of memories to extract from the trace

          format: Trace format: 'vercel', 'hyperdoc', or 'openclaw'. Auto-detected if not set.

          metadata: Custom metadata for filtering. Keys must be alphanumeric with underscores, max
              64 chars.

          session_id: Resource identifier for the trace.

          title: Title of the trace

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/trace/add",
            body=await async_maybe_transform(
                {
                    "history": history,
                    "date": date,
                    "extract": extract,
                    "format": format,
                    "metadata": metadata,
                    "session_id": session_id,
                    "title": title,
                },
                session_add_params.SessionAddParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryStatus,
        )


class SessionsResourceWithRawResponse:
    def __init__(self, sessions: SessionsResource) -> None:
        self._sessions = sessions

        self.add = to_raw_response_wrapper(
            sessions.add,
        )


class AsyncSessionsResourceWithRawResponse:
    def __init__(self, sessions: AsyncSessionsResource) -> None:
        self._sessions = sessions

        self.add = async_to_raw_response_wrapper(
            sessions.add,
        )


class SessionsResourceWithStreamingResponse:
    def __init__(self, sessions: SessionsResource) -> None:
        self._sessions = sessions

        self.add = to_streamed_response_wrapper(
            sessions.add,
        )


class AsyncSessionsResourceWithStreamingResponse:
    def __init__(self, sessions: AsyncSessionsResource) -> None:
        self._sessions = sessions

        self.add = async_to_streamed_response_wrapper(
            sessions.add,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import action_add_reaction_params, action_send_message_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.action_add_reaction_response import ActionAddReactionResponse
from ..types.action_send_message_response import ActionSendMessageResponse

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    def add_reaction(
        self,
        *,
        channel: str,
        name: str,
        provider: Literal[
            "reddit",
            "notion",
            "slack",
            "google_calendar",
            "google_mail",
            "box",
            "dropbox",
            "google_drive",
            "github",
            "vault",
            "web_crawler",
            "trace",
            "microsoft_teams",
        ],
        timestamp: str,
        connection: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionAddReactionResponse:
        """
        Add an emoji reaction to a message on a connected integration.

        Args:
          channel: Channel ID containing the message

          name: Emoji name without colons (e.g., thumbsup)

          provider: Integration provider (e.g., slack)

          timestamp: Message timestamp to react to

          connection: Connection ID. If omitted, auto-resolved from provider + user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/actions/add_reaction",
            body=maybe_transform(
                {
                    "channel": channel,
                    "name": name,
                    "provider": provider,
                    "timestamp": timestamp,
                    "connection": connection,
                },
                action_add_reaction_params.ActionAddReactionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionAddReactionResponse,
        )

    def send_message(
        self,
        *,
        provider: Literal[
            "reddit",
            "notion",
            "slack",
            "google_calendar",
            "google_mail",
            "box",
            "dropbox",
            "google_drive",
            "github",
            "vault",
            "web_crawler",
            "trace",
            "microsoft_teams",
        ],
        text: str,
        channel: Optional[str] | Omit = omit,
        connection: Optional[str] | Omit = omit,
        parent: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSendMessageResponse:
        """
        Send a message to a channel or conversation on a connected integration.

        Args:
          provider: Integration provider (e.g., slack)

          text: Message text

          channel: Channel ID (required for Slack)

          connection: Connection ID. If omitted, auto-resolved from provider + user.

          parent: Parent message ID for threading (thread_ts for Slack)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/actions/send_message",
            body=maybe_transform(
                {
                    "provider": provider,
                    "text": text,
                    "channel": channel,
                    "connection": connection,
                    "parent": parent,
                },
                action_send_message_params.ActionSendMessageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSendMessageResponse,
        )


class AsyncActionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    async def add_reaction(
        self,
        *,
        channel: str,
        name: str,
        provider: Literal[
            "reddit",
            "notion",
            "slack",
            "google_calendar",
            "google_mail",
            "box",
            "dropbox",
            "google_drive",
            "github",
            "vault",
            "web_crawler",
            "trace",
            "microsoft_teams",
        ],
        timestamp: str,
        connection: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionAddReactionResponse:
        """
        Add an emoji reaction to a message on a connected integration.

        Args:
          channel: Channel ID containing the message

          name: Emoji name without colons (e.g., thumbsup)

          provider: Integration provider (e.g., slack)

          timestamp: Message timestamp to react to

          connection: Connection ID. If omitted, auto-resolved from provider + user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/actions/add_reaction",
            body=await async_maybe_transform(
                {
                    "channel": channel,
                    "name": name,
                    "provider": provider,
                    "timestamp": timestamp,
                    "connection": connection,
                },
                action_add_reaction_params.ActionAddReactionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionAddReactionResponse,
        )

    async def send_message(
        self,
        *,
        provider: Literal[
            "reddit",
            "notion",
            "slack",
            "google_calendar",
            "google_mail",
            "box",
            "dropbox",
            "google_drive",
            "github",
            "vault",
            "web_crawler",
            "trace",
            "microsoft_teams",
        ],
        text: str,
        channel: Optional[str] | Omit = omit,
        connection: Optional[str] | Omit = omit,
        parent: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSendMessageResponse:
        """
        Send a message to a channel or conversation on a connected integration.

        Args:
          provider: Integration provider (e.g., slack)

          text: Message text

          channel: Channel ID (required for Slack)

          connection: Connection ID. If omitted, auto-resolved from provider + user.

          parent: Parent message ID for threading (thread_ts for Slack)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/actions/send_message",
            body=await async_maybe_transform(
                {
                    "provider": provider,
                    "text": text,
                    "channel": channel,
                    "connection": connection,
                    "parent": parent,
                },
                action_send_message_params.ActionSendMessageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSendMessageResponse,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.add_reaction = to_raw_response_wrapper(
            actions.add_reaction,
        )
        self.send_message = to_raw_response_wrapper(
            actions.send_message,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.add_reaction = async_to_raw_response_wrapper(
            actions.add_reaction,
        )
        self.send_message = async_to_raw_response_wrapper(
            actions.send_message,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.add_reaction = to_streamed_response_wrapper(
            actions.add_reaction,
        )
        self.send_message = to_streamed_response_wrapper(
            actions.send_message,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.add_reaction = async_to_streamed_response_wrapper(
            actions.add_reaction,
        )
        self.send_message = async_to_streamed_response_wrapper(
            actions.send_message,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from .._models import BaseModel

__all__ = ["ActionSendMessageResponse"]


class ActionSendMessageResponse(BaseModel):
    """Result from executing an integration action."""

    success: bool

    error: Optional[str] = None

    provider_response: Optional[Dict[str, object]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Literal, Required, TypedDict

__all__ = ["MemoryUpdateParams"]


class MemoryUpdateParams(TypedDict, total=False):
    source: Required[
        Literal[
            "reddit",
            "notion",
            "slack",
            "google_calendar",
            "google_mail",
            "box",
            "dropbox",
            "google_drive",
            "github",
            "vault",
            "web_crawler",
            "trace",
            "microsoft_teams",
        ]
    ]

    collection: Union[str, object, None]
    """
    The collection to move the document to — deprecated, set the collection using
    metadata instead.
    """

    metadata: Union[Dict[str, Union[str, float, bool, None]], object, None]
    """Custom metadata for filtering.

    Keys must be alphanumeric with underscores, max 64 chars. Values must be string,
    number, boolean, or null. Will be merged with existing metadata.
    """

    text: Union[str, object, None]
    """Full text of the document. If provided, the document will be re-indexed."""

    title: Union[str, object, None]
    """Title of the document."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["IntegrationListResponse", "Integration"]


class Integration(BaseModel):
    id: str
    """The integration's id"""

    allow_multiple_connections: bool
    """Whether the integration allows multiple connections"""

    auth_provider: Literal["nango", "unified", "whitelabel"]
    """The integration's auth provider"""

    icon: Optional[str] = None
    """URL to the integration's icon"""

    name: str
    """The integration's display name"""

    provider: Literal[
        "reddit",
        "notion",
        "slack",
        "google_calendar",
        "google_mail",
        "box",
        "dropbox",
        "google_drive",
        "github",
        "vault",
        "web_crawler",
        "trace",
        "microsoft_teams",
    ]
    """The integration's provider"""


class IntegrationListResponse(BaseModel):
    integrations: List[Integration]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hyperspell import Hyperspell, AsyncHyperspell
from tests.utils import assert_matches_type
from hyperspell.types import (
    ActionAddReactionResponse,
    ActionSendMessageResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestActions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_add_reaction(self, client: Hyperspell) -> None:
        action = client.actions.add_reaction(
            channel="channel",
            name="name",
            provider="reddit",
            timestamp="timestamp",
        )
        assert_matches_type(ActionAddReactionResponse, action, path=["response"])

    @parametrize
    def test_method_add_reaction_with_all_params(self, client: Hyperspell) -> None:
        action = client.actions.add_reaction(
            channel="channel",
            name="name",
            provider="reddit",
            timestamp="timestamp",
            connection="connection",
        )
        assert_matches_type(ActionAddReactionResponse, action, path=["response"])

    @parametrize
    def test_raw_response_add_reaction(self, client: Hyperspell) -> None:
        response = client.actions.with_raw_response.add_reaction(
            channel="channel",
            name="name",
            provider="reddit",
            timestamp="timestamp",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(ActionAddReactionResponse, action, path=["response"])

    @parametrize
    def test_streaming_response_add_reaction(self, client: Hyperspell) -> None:
        with client.actions.with_streaming_response.add_reaction(
            channel="channel",
            name="name",
            provider="reddit",
            timestamp="timestamp",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(ActionAddReactionResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_send_message(self, client: Hyperspell) -> None:
        action = client.actions.send_message(
            provider="reddit",
            text="text",
        )
        assert_matches_type(ActionSendMessageResponse, action, path=["response"])

    @parametrize
    def test_method_send_message_with_all_params(self, client: Hyperspell) -> None:
        action = client.actions.send_message(
            provider="reddit",
            text="text",
            channel="channel",
            connection="connection",
            parent="parent",
        )
        assert_matches_type(ActionSendMessageResponse, action, path=["response"])

    @parametrize
    def test_raw_response_send_message(self, client: Hyperspell) -> None:
        response = client.actions.with_raw_response.send_message(
            provider="reddit",
            text="text",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(ActionSendMessageResponse, action, path=["response"])

    @parametrize
    def test_streaming_response_send_message(self, client: Hyperspell) -> None:
        with client.actions.with_streaming_response.send_message(
            provider="reddit",
            text="text",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(ActionSendMessageResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncActions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_add_reaction(self, async_client: AsyncHyperspell) -> None:
        action = await async_client.actions.add_reaction(
            channel="channel",
            name="name",
            provider="reddit",
            timestamp="timestamp",
        )
        assert_matches_type(ActionAddReactionResponse, action, path=["response"])

    @parametrize
    async def test_method_add_reaction_with_all_params(self, async_client: AsyncHyperspell) -> None:
        action = await async_client.actions.add_reaction(
            channel="channel",
            name="name",
            provider="reddit",
            timestamp="timestamp",
            connection="connection",
        )
        assert_matches_type(ActionAddReactionResponse, action, path=["response"])

    @parametrize
    async def test_raw_response_add_reaction(self, async_client: AsyncHyperspell) -> None:
        response = await async_client.actions.with_raw_response.add_reaction(
            channel="channel",
            name="name",
            provider="reddit",
            timestamp="timestamp",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(ActionAddReactionResponse, action, path=["response"])

    @parametrize
    async def test_streaming_response_add_reaction(self, async_client: AsyncHyperspell) -> None:
        async with async_client.actions.with_streaming_response.add_reaction(
            channel="channel",
            name="name",
            provider="reddit",
            timestamp="timestamp",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(ActionAddReactionResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_send_message(self, async_client: AsyncHyperspell) -> None:
        action = await async_client.actions.send_message(
            provider="reddit",
            text="text",
        )
        assert_matches_type(ActionSendMessageResponse, action, path=["response"])

    @parametrize
    async def test_method_send_message_with_all_params(self, async_client: AsyncHyperspell) -> None:
        action = await async_client.actions.send_message(
            provider="reddit",
            text="text",
            channel="channel",
            connection="connection",
            parent="parent",
        )
        assert_matches_type(ActionSendMessageResponse, action, path=["response"])

    @parametrize
    async def test_raw_response_send_message(self, async_client: AsyncHyperspell) -> None:
        response = await async_client.actions.with_raw_response.send_message(
            provider="reddit",
            text="text",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(ActionSendMessageResponse, action, path=["response"])

    @parametrize
    async def test_streaming_response_send_message(self, async_client: AsyncHyperspell) -> None:
        async with async_client.actions.with_streaming_response.send_message(
            provider="reddit",
            text="text",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(ActionSendMessageResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ConnectionListResponse", "Connection"]


class Connection(BaseModel):
    id: str
    """The connection's id"""

    integration_id: str
    """The connection's integration id"""

    label: Optional[str] = None
    """The connection's label"""

    provider: Literal[
        "reddit",
        "notion",
        "slack",
        "google_calendar",
        "google_mail",
        "box",
        "dropbox",
        "google_drive",
        "github",
        "vault",
        "web_crawler",
        "trace",
        "microsoft_teams",
    ]
    """The connection's provider"""


class ConnectionListResponse(BaseModel):
    connections: List[Connection]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ActionSendMessageParams"]


class ActionSendMessageParams(TypedDict, total=False):
    provider: Required[
        Literal[
            "reddit",
            "notion",
            "slack",
            "google_calendar",
            "google_mail",
            "box",
            "dropbox",
            "google_drive",
            "github",
            "vault",
            "web_crawler",
            "trace",
            "microsoft_teams",
        ]
    ]
    """Integration provider (e.g., slack)"""

    text: Required[str]
    """Message text"""

    channel: Optional[str]
    """Channel ID (required for Slack)"""

    connection: Optional[str]
    """Connection ID. If omitted, auto-resolved from provider + user."""

    parent: Optional[str]
    """Parent message ID for threading (thread_ts for Slack)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["WebCrawlerIndexResponse"]


class WebCrawlerIndexResponse(BaseModel):
    resource_id: str

    source: Literal[
        "reddit",
        "notion",
        "slack",
        "google_calendar",
        "google_mail",
        "box",
        "dropbox",
        "google_drive",
        "github",
        "vault",
        "web_crawler",
        "trace",
        "microsoft_teams",
    ]

    status: Literal["pending", "processing", "completed", "failed"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, HyperspellError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import auth, vaults, actions, evaluate, memories, sessions, connections, integrations
    from .resources.auth import AuthResource, AsyncAuthResource
    from .resources.vaults import VaultsResource, AsyncVaultsResource
    from .resources.actions import ActionsResource, AsyncActionsResource
    from .resources.evaluate import EvaluateResource, AsyncEvaluateResource
    from .resources.memories import MemoriesResource, AsyncMemoriesResource
    from .resources.sessions import SessionsResource, AsyncSessionsResource
    from .resources.connections import ConnectionsResource, AsyncConnectionsResource
    from .resources.integrations.integrations import IntegrationsResource, AsyncIntegrationsResource

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Hyperspell",
    "AsyncHyperspell",
    "Client",
    "AsyncClient",
]


class Hyperspell(SyncAPIClient):
    # client options
    api_key: str
    user_id: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        user_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Hyperspell client instance.

        This automatically infers the `api_key` argument from the `HYPERSPELL_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("HYPERSPELL_API_KEY")
        if api_key is None:
            raise HyperspellError(
                "The api_key client option must be set either by passing api_key to the client or by setting the HYPERSPELL_API_KEY environment variable"
            )
        self.api_key = api_key

        self.user_id = user_id

        if base_url is None:
            base_url = os.environ.get("HYPERSPELL_BASE_URL")
        if base_url is None:
            base_url = f"https://api.hyperspell.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def connections(self) -> ConnectionsResource:
        from .resources.connections import ConnectionsResource

        return ConnectionsResource(self)

    @cached_property
    def integrations(self) -> IntegrationsResource:
        from .resources.integrations import IntegrationsResource

        return IntegrationsResource(self)

    @cached_property
    def memories(self) -> MemoriesResource:
        from .resources.memories import MemoriesResource

        return MemoriesResource(self)

    @cached_property
    def evaluate(self) -> EvaluateResource:
        from .resources.evaluate import EvaluateResource

        return EvaluateResource(self)

    @cached_property
    def actions(self) -> ActionsResource:
        from .resources.actions import ActionsResource

        return ActionsResource(self)

    @cached_property
    def sessions(self) -> SessionsResource:
        from .resources.sessions import SessionsResource

        return SessionsResource(self)

    @cached_property
    def vaults(self) -> VaultsResource:
        from .resources.vaults import VaultsResource

        return VaultsResource(self)

    @cached_property
    def auth(self) -> AuthResource:
        from .resources.auth import AuthResource

        return AuthResource(self)

    @cached_property
    def with_raw_response(self) -> HyperspellWithRawResponse:
        return HyperspellWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> HyperspellWithStreamedResponse:
        return HyperspellWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        return {**self._api_key, **self._as_user}

    @property
    def _api_key(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    def _as_user(self) -> dict[str, str]:
        user_id = self.user_id
        if user_id is None:
            return {}
        return {"X-As-User": user_id}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        user_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            user_id=user_id or self.user_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncHyperspell(AsyncAPIClient):
    # client options
    api_key: str
    user_id: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        user_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncHyperspell client instance.

        This automatically infers the `api_key` argument from the `HYPERSPELL_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("HYPERSPELL_API_KEY")
        if api_key is None:
            raise HyperspellError(
                "The api_key client option must be set either by passing api_key to the client or by setting the HYPERSPELL_API_KEY environment variable"
            )
        self.api_key = api_key

        self.user_id = user_id

        if base_url is None:
            base_url = os.environ.get("HYPERSPELL_BASE_URL")
        if base_url is None:
            base_url = f"https://api.hyperspell.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def connections(self) -> AsyncConnectionsResource:
        from .resources.connections import AsyncConnectionsResource

        return AsyncConnectionsResource(self)

    @cached_property
    def integrations(self) -> AsyncIntegrationsResource:
        from .resources.integrations import AsyncIntegrationsResource

        return AsyncIntegrationsResource(self)

    @cached_property
    def memories(self) -> AsyncMemoriesResource:
        from .resources.memories import AsyncMemoriesResource

        return AsyncMemoriesResource(self)

    @cached_property
    def evaluate(self) -> AsyncEvaluateResource:
        from .resources.evaluate import AsyncEvaluateResource

        return AsyncEvaluateResource(self)

    @cached_property
    def actions(self) -> AsyncActionsResource:
        from .resources.actions import AsyncActionsResource

        return AsyncActionsResource(self)

    @cached_property
    def sessions(self) -> AsyncSessionsResource:
        from .resources.sessions import AsyncSessionsResource

        return AsyncSessionsResource(self)

    @cached_property
    def vaults(self) -> AsyncVaultsResource:
        from .resources.vaults import AsyncVaultsResource

        return AsyncVaultsResource(self)

    @cached_property
    def auth(self) -> AsyncAuthResource:
        from .resources.auth import AsyncAuthResource

        return AsyncAuthResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncHyperspellWithRawResponse:
        return AsyncHyperspellWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncHyperspellWithStreamedResponse:
        return AsyncHyperspellWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        return {**self._api_key, **self._as_user}

    @property
    def _api_key(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    def _as_user(self) -> dict[str, str]:
        user_id = self.user_id
        if user_id is None:
            return {}
        return {"X-As-User": user_id}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        user_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            user_id=user_id or self.user_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class HyperspellWithRawResponse:
    _client: Hyperspell

    def __init__(self, client: Hyperspell) -> None:
        self._client = client

    @cached_property
    def connections(self) -> connections.ConnectionsResourceWithRawResponse:
        from .resources.connections import ConnectionsResourceWithRawResponse

        return ConnectionsResourceWithRawResponse(self._client.connections)

    @cached_property
    def integrations(self) -> integrations.IntegrationsResourceWithRawResponse:
        from .resources.integrations import IntegrationsResourceWithRawResponse

        return IntegrationsResourceWithRawResponse(self._client.integrations)

    @cached_property
    def memories(self) -> memories.MemoriesResourceWithRawResponse:
        from .resources.memories import MemoriesResourceWithRawResponse

        return MemoriesResourceWithRawResponse(self._client.memories)

    @cached_property
    def evaluate(self) -> evaluate.EvaluateResourceWithRawResponse:
        from .resources.evaluate import EvaluateResourceWithRawResponse

        return EvaluateResourceWithRawResponse(self._client.evaluate)

    @cached_property
    def actions(self) -> actions.ActionsResourceWithRawResponse:
        from .resources.actions import ActionsResourceWithRawResponse

        return ActionsResourceWithRawResponse(self._client.actions)

    @cached_property
    def sessions(self) -> sessions.SessionsResourceWithRawResponse:
        from .resources.sessions import SessionsResourceWithRawResponse

        return SessionsResourceWithRawResponse(self._client.sessions)

    @cached_property
    def vaults(self) -> vaults.VaultsResourceWithRawResponse:
        from .resources.vaults import VaultsResourceWithRawResponse

        return VaultsResourceWithRawResponse(self._client.vaults)

    @cached_property
    def auth(self) -> auth.AuthResourceWithRawResponse:
        from .resources.auth import AuthResourceWithRawResponse

        return AuthResourceWithRawResponse(self._client.auth)


class AsyncHyperspellWithRawResponse:
    _client: AsyncHyperspell

    def __init__(self, client: AsyncHyperspell) -> None:
        self._client = client

    @cached_property
    def connections(self) -> connections.AsyncConnectionsResourceWithRawResponse:
        from .resources.connections import AsyncConnectionsResourceWithRawResponse

        return AsyncConnectionsResourceWithRawResponse(self._client.connections)

    @cached_property
    def integrations(self) -> integrations.AsyncIntegrationsResourceWithRawResponse:
        from .resources.integrations import AsyncIntegrationsResourceWithRawResponse

        return AsyncIntegrationsResourceWithRawResponse(self._client.integrations)

    @cached_property
    def memories(self) -> memories.AsyncMemoriesResourceWithRawResponse:
        from .resources.memories import AsyncMemoriesResourceWithRawResponse

        return AsyncMemoriesResourceWithRawResponse(self._client.memories)

    @cached_property
    def evaluate(self) -> evaluate.AsyncEvaluateResourceWithRawResponse:
        from .resources.evaluate import AsyncEvaluateResourceWithRawResponse

        return AsyncEvaluateResourceWithRawResponse(self._client.evaluate)

    @cached_property
    def actions(self) -> actions.AsyncActionsResourceWithRawResponse:
        from .resources.actions import AsyncActionsResourceWithRawResponse

        return AsyncActionsResourceWithRawResponse(self._client.actions)

    @cached_property
    def sessions(self) -> sessions.AsyncSessionsResourceWithRawResponse:
        from .resources.sessions import AsyncSessionsResourceWithRawResponse

        return AsyncSessionsResourceWithRawResponse(self._client.sessions)

    @cached_property
    def vaults(self) -> vaults.AsyncVaultsResourceWithRawResponse:
        from .resources.vaults import AsyncVaultsResourceWithRawResponse

        return AsyncVaultsResourceWithRawResponse(self._client.vaults)

    @cached_property
    def auth(self) -> auth.AsyncAuthResourceWithRawResponse:
        from .resources.auth import AsyncAuthResourceWithRawResponse

        return AsyncAuthResourceWithRawResponse(self._client.auth)


class HyperspellWithStreamedResponse:
    _client: Hyperspell

    def __init__(self, client: Hyperspell) -> None:
        self._client = client

    @cached_property
    def connections(self) -> connections.ConnectionsResourceWithStreamingResponse:
        from .resources.connections import ConnectionsResourceWithStreamingResponse

        return ConnectionsResourceWithStreamingResponse(self._client.connections)

    @cached_property
    def integrations(self) -> integrations.IntegrationsResourceWithStreamingResponse:
        from .resources.integrations import IntegrationsResourceWithStreamingResponse

        return IntegrationsResourceWithStreamingResponse(self._client.integrations)

    @cached_property
    def memories(self) -> memories.MemoriesResourceWithStreamingResponse:
        from .resources.memories import MemoriesResourceWithStreamingResponse

        return MemoriesResourceWithStreamingResponse(self._client.memories)

    @cached_property
    def evaluate(self) -> evaluate.EvaluateResourceWithStreamingResponse:
        from .resources.evaluate import EvaluateResourceWithStreamingResponse

        return EvaluateResourceWithStreamingResponse(self._client.evaluate)

    @cached_property
    def actions(self) -> actions.ActionsResourceWithStreamingResponse:
        from .resources.actions import ActionsResourceWithStreamingResponse

        return ActionsResourceWithStreamingResponse(self._client.actions)

    @cached_property
    def sessions(self) -> sessions.SessionsResourceWithStreamingResponse:
        from .resources.sessions import SessionsResourceWithStreamingResponse

        return SessionsResourceWithStreamingResponse(self._client.sessions)

    @cached_property
    def vaults(self) -> vaults.VaultsResourceWithStreamingResponse:
        from .resources.vaults import VaultsResourceWithStreamingResponse

        return VaultsResourceWithStreamingResponse(self._client.vaults)

    @cached_property
    def auth(self) -> auth.AuthResourceWithStreamingResponse:
        from .resources.auth import AuthResourceWithStreamingResponse

        return AuthResourceWithStreamingResponse(self._client.auth)


class AsyncHyperspellWithStreamedResponse:
    _client: AsyncHyperspell

    def __init__(self, client: AsyncHyperspell) -> None:
        self._client = client

    @cached_property
    def connections(self) -> connections.AsyncConnectionsResourceWithStreamingResponse:
        from .resources.connections import AsyncConnectionsResourceWithStreamingResponse

        return AsyncConnectionsResourceWithStreamingResponse(self._client.connections)

    @cached_property
    def integrations(self) -> integrations.AsyncIntegrationsResourceWithStreamingResponse:
        from .resources.integrations import AsyncIntegrationsResourceWithStreamingResponse

        return AsyncIntegrationsResourceWithStreamingResponse(self._client.integrations)

    @cached_property
    def memories(self) -> memories.AsyncMemoriesResourceWithStreamingResponse:
        from .resources.memories import AsyncMemoriesResourceWithStreamingResponse

        return AsyncMemoriesResourceWithStreamingResponse(self._client.memories)

    @cached_property
    def evaluate(self) -> evaluate.AsyncEvaluateResourceWithStreamingResponse:
        from .resources.evaluate import AsyncEvaluateResourceWithStreamingResponse

        return AsyncEvaluateResourceWithStreamingResponse(self._client.evaluate)

    @cached_property
    def actions(self) -> actions.AsyncActionsResourceWithStreamingResponse:
        from .resources.actions import AsyncActionsResourceWithStreamingResponse

        return AsyncActionsResourceWithStreamingResponse(self._client.actions)

    @cached_property
    def sessions(self) -> sessions.AsyncSessionsResourceWithStreamingResponse:
        from .resources.sessions import AsyncSessionsResourceWithStreamingResponse

        return AsyncSessionsResourceWithStreamingResponse(self._client.sessions)

    @cached_property
    def vaults(self) -> vaults.AsyncVaultsResourceWithStreamingResponse:
        from .resources.vaults import AsyncVaultsResourceWithStreamingResponse

        return AsyncVaultsResourceWithStreamingResponse(self._client.vaults)

    @cached_property
    def auth(self) -> auth.AsyncAuthResourceWithStreamingResponse:
        from .resources.auth import AsyncAuthResourceWithStreamingResponse

        return AsyncAuthResourceWithStreamingResponse(self._client.auth)


Client = Hyperspell

AsyncClient = AsyncHyperspell
